/*
 * Copyright (C) 2013 - Juan Ferrer Toribio
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "vn-users.h"
#include "stdlib.h"

G_DEFINE_TYPE (VnUsers, vn_users, VN_TYPE_FORM);

//+++++++++++++++++++++++++++++++++++++++++++++++++++ Private

static void vn_users_reset_dialog (VnUsers * obj, const gchar * error)
{
	GtkWidget * infobar = vn_form_get (VN_FORM (obj), "password-infobar");

	if (error)
	{
		GtkLabel * password_error = vn_form_get (VN_FORM (obj), "password-error");
		gtk_label_set_text (password_error, error);
		gtk_widget_show (infobar);
	}
	else
		gtk_widget_hide (infobar);

	gtk_entry_set_text (obj->repeat_password, "");
	gtk_entry_set_text (obj->password_entry, "");
	gtk_widget_grab_focus (GTK_WIDGET (obj->password_entry));
}

void vn_users_on_set_password_clicked (GtkButton * button, VnUsers * obj)
{
	gtk_window_set_transient_for (GTK_WINDOW (obj->password_dialog),
		GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (obj))));
	gtk_widget_show (GTK_WIDGET (obj->password_dialog));
	vn_users_reset_dialog (obj, NULL);
}

static void vn_users_on_password_changed (DbRequest * request, VnUsers * obj)
{
	GValue value = {0};
	
	if (db_request_fetch_value (request, &value, NULL))
	{
		if (g_value_get_int (&value) != -1)
			gtk_widget_hide (GTK_WIDGET (obj->password_dialog));
		else
			vn_users_reset_dialog (obj, _("The password is too week."));
		
		g_value_unset (&value);
	}

	g_object_unref (request);
}

void vn_users_on_dialog_response (GtkDialog * dialog, gint response_id, VnUsers * obj)
{
	if (response_id == GTK_RESPONSE_ACCEPT)
	{
		const gchar * password = gtk_entry_get_text (obj->password_entry);

		if (!g_strcmp0 (password, ""))
			vn_users_reset_dialog (obj, _("The password can't be empty."));
		else if (g_strcmp0 (password, gtk_entry_get_text (obj->repeat_password)))
			vn_users_reset_dialog (obj, _("Passwords do not match."));
		else
		{
			SqlObject * query;
			SqlBatch * batch;
			
			query = sql_string_new ("SELECT user_set_password (#user, #password)");
			
			batch = sql_batch_new ();
			sql_batch_add_from_param (batch, "user", obj->user_id); 
			sql_batch_add_from_value (batch, "password", G_TYPE_STRING, password);

			db_conn_query_with_stmt_async (VN_FORM (obj)->conn
				,SQL_STMT (query)
				,batch
				,(DbRequestDoneCallback) vn_users_on_password_changed
				,g_object_ref (obj)
				,g_object_unref
			);
		}
	}
	else
		gtk_widget_hide (GTK_WIDGET (dialog));
}

static void vn_users_open (VnUsers * obj, gpointer user_data)
{
	DbIterator * users;
	VnForm * form = VN_FORM (obj);

	obj->password_entry = vn_form_get (form, "password-entry");
	obj->repeat_password = vn_form_get (form, "repeat-password");

	obj->password_dialog = vn_form_get (form, "password-dialog");
	gtk_dialog_add_buttons (obj->password_dialog
		,_("Cancel"), GTK_RESPONSE_CANCEL
		,_("Apply"), GTK_RESPONSE_ACCEPT
		,NULL
	);

	users = vn_form_get (form, "users");
	obj->user_id = db_iterator_get_param (users, "id");
	
	obj->account = vn_form_get (form, "account");
	db_iterator_link_with_param (obj->account, "user_id", obj->user_id);
	
	obj->alias = vn_form_get (form, "alias");
	db_iterator_link (obj->alias, "user_id", obj->account, "user_id");	

	obj->sip = vn_form_get (form, "sip");
	db_iterator_link (obj->sip, "user_id", obj->account, "user_id");
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++ Class

static void vn_users_init (VnUsers * obj) {}

static void vn_users_class_init (VnUsersClass * k)
{
	VN_FORM_CLASS (k)->open = (VnFormOpenFunc) vn_users_open;
}
